/**
 * HSVColorTracking
 * Greg Borenstein
 * https://github.com/atduskgreg/opencv-processing-book/blob/master/code/hsv_color_tracking/HSVColorTracking/HSVColorTracking.pde
 *
 * Modified by Jordi Tost @jorditost (color selection)
 *
 * University of Applied Sciences Potsdam, 2014
 */

import gab.opencv.*;
import processing.video.*;
import java.awt.Rectangle;

Capture video;
OpenCV opencv;
PImage src, colorFilteredImage, deImage;
ArrayList<Contour> contours;

import themidibus.*; //Import the library

MidiBus myBus; // The MidiBus

// <1> Set the range of Hue values for our filter
int rangeLow = 15;
int rangeHigh = 40;

int xpos = 0;
int ypos = 680;
int prevY = 0;
int xStep = 5;
int yOrigin = 0;
int currY = 0;
boolean drawTime = false;

int prevTime = 0;

int cameraNum = 0; // Pick which camera to use

int erodeNum = 4;
int dilateNum = 1;
int contourNum = 3; // MAximum 3

// MIDI Stuff
int[] channel = {0,1,2};
int[] pitch = {36,32,44};
int velocity = 110;
int[] cc1 = {40,41,42};
int[] cc2 = {50,51,52};
int[] cc1val = {0,0,0};
int[] cc2val = {0,0,0};
color c1 = color(155,10,10);
color c2 = color(97,181,10);
color c3 = color(171,142,190);
color c4 = color(172,209,103);
color c5 = color(94,192,156);
color c6 = color(255,179,66);
 
color[] colors  = {  
  c1, c2, c3, c4, c5, c6
};

void setup() {
  String[] cameras = Capture.list();

  if (cameras.length == 0) {
    println("There are no cameras available for capture.");
    exit();
  } else {
    println("Available cameras:");
    for (int i = 0; i < cameras.length; i++) {
      print(i);
      print(" ");
      println(cameras[i]);
    }
  }
  video = new Capture(this, 640, 480, cameras[cameraNum]);
  video.start();

  opencv = new OpenCV(this, video.width, video.height);
  contours = new ArrayList<Contour>();
  
  size(1280, 480, P2D);
  // https://help.ableton.com/hc/en-us/articles/209774225-Using-virtual-MIDI-buses-in-Live
  MidiBus.list(); // List all available Midi devices on STDOUT. This will show each device's index and name.
  myBus = new MidiBus(this, -1, "vMIDI"); // Create a new MidiBus with no input device
}

void draw() {
  
  // Read last captured frame
  if (video.available()) {
    video.read();
  }

  // <2> Load the new frame of our movie in to OpenCV
  opencv.loadImage(video);
  
  // Tell OpenCV to use color information
  opencv.useColor();
  src = opencv.getSnapshot();
  
  // <3> Tell OpenCV to work in HSV color space.
  opencv.useColor(HSB);
  
  // <4> Copy the Hue channel of our image into 
  //     the gray channel, which we process.
  opencv.setGray(opencv.getH().clone());
  
  // <5> Filter the image based on the range of 
  //     hue values that match the object we want to track.
  opencv.inRange(rangeLow, rangeHigh);
  
  // <6> erode and dilate and the image to remove artifacts and noise 
  
  for(int n = 0; n < erodeNum; n++){
    opencv.erode();
  }
  for(int n = 0; n < dilateNum; n++){
    opencv.dilate();
  }
 
  // <7> Get the processed image for reference.
  
  colorFilteredImage = opencv.getSnapshot();
  ///////////////////////////////////////////
  // We could process our image here!
  // See ImageFiltering.pde
  ///////////////////////////////////////////
  
  // <7> Find contours in our range image.
  //     Passing 'true' sorts them by descending area.
  contours = opencv.findContours(true, true);
  
  // <8> Display background images
  image(src, 0, 0);
  image(colorFilteredImage, src.width, 0);
  
  // <9> Check to make sure we've found any contours
  if (contours.size() > 0) {
    int cons = contours.size();
    if(contours.size() > contourNum){
      cons = contourNum;
    }
    for(int n = 0; n < cons; n++){
      // <9> Get the first contour, which will be the largest one
      Contour biggestContour = contours.get(n);
      
      // <10> Find the bounding box of the largest contour,
      //      and hence our object.
      Rectangle r = biggestContour.getBoundingBox();
      
      // <11> Draw the bounding box of our object
      noFill(); 
      strokeWeight(2); 
      stroke(colors[n]);
      rect(r.x, r.y, r.width, r.height);
      
      // <12> Draw a dot in the middle of the bounding box, on the object.
      noStroke(); 
      fill(colors[n]);
      ellipse(r.x + r.width/2, r.y + r.height/2, 30, 30);    
      cc1val[n] = int(map(r.x + r.width/2,0,680,0,127));
      cc2val[n] = int(map(r.y + r.height/2,0,480,127,0));
      myBus.sendControllerChange(channel[n], cc1[n], cc1val[n]); // Send a controllerChange
      if(n == 0){
        pitchBend(int(map(cc2val[n],0,127,0,16383))); // Send a pitch bend
      } else{
        myBus.sendControllerChange(channel[n], cc2[n], cc2val[n]); // Send a controllerChange      
      }
    }
  }
}

void pitchBend(int pb){
  
  // The pitch bend value is a 14-bit number (0-16383). 0x2000 (8192) is the default / middle value.

  int status_byte = 224; // Pitchbend 224
  int channel_byte = channel[0]; // Channel
  int first_byte = pb & 0x7F; // The same note;
  int second_byte = pb >> 7; // But with less velocity

  myBus.sendMessage(status_byte, channel_byte, first_byte, second_byte);
}

void mousePressed(){
  for(int n = 0; n < contourNum; n++){
    myBus.sendNoteOff(channel[n], pitch[n], 0); // Send a Midi noteOff
    myBus.sendNoteOn(channel[n], pitch[n], velocity); // Send a Midi noteOn  
  }
  
  //color c = get(mouseX, mouseY);
  color c = getAverageRGBCircle(src, mouseX, mouseY, 5);
  
  println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
   
  int hue = int(map(hue(c), 0, 255, 0, 180));
  println("hue to detect: " + hue);
  rangeLow = hue - 10;
  rangeHigh = hue + 10;
}

void keyPressed(){
// When the key is pressed slowly send a control change - for MIDI mapping...
  if (key=='1') {
     delay(2000);
     myBus.sendControllerChange(channel[0], cc1[0], 0); // Send a controllerChange
     delay(2000);
  } else if(key=='2') {
     delay(2000); 
     myBus.sendControllerChange(channel[0], cc2[0], 0); // Send a controllerChange
     delay(2000);
  } else if(key=='3') {
     delay(2000);
     myBus.sendControllerChange(channel[1], cc1[1], 0); // Send a controllerChange
     delay(2000);
  } else if(key=='4') {
     delay(2000); 
     myBus.sendControllerChange(channel[1], cc2[1], 0); // Send a controllerChange
     delay(2000);
  } else if(key=='5') {
     delay(2000); 
     myBus.sendControllerChange(channel[2], cc1[2], 0); // Send a controllerChange
     delay(2000);
  } else if(key=='6') {
     delay(2000); 
     myBus.sendControllerChange(channel[2], cc2[2], 0); // Send a controllerChange
     delay(2000);  
  }
}

color getAverageRGBCircle(PImage img, int x, int y, int radius) {
  float r = 0;
  float g = 0;
  float b = 0;
  int num = 0;
  /* Iterate through a bounding box in which the circle lies */
  for (int i = x - radius; i < x + radius; i++) {
    for (int j = y - radius; j < y + radius; j++) {
      /* If the pixel is outside the canvas, skip it */
      if (i < 0 || i >= width || j < 0 || j >= height)
        continue;

      /* If the pixel is outside the circle, skip it */
      if (dist(x, y, i, j) > r)
        continue;

      /* Get the color from the image, add to a running sum */
      color c = img.get(i, j);
      r += red(c);
      g += green(c);
      b += blue(c);
      num++;
    }
  }
  /* Return the mean of the R, G, and B components */
  return color(r/num, g/num, b/num);
}
